/*
 * Klingel.c
 *
 * Created: 19.03.2023 18:54:31
 * Author : Alexa
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <math.h>
#include <avr/wdt.h>
#include "adc.h"
#include "Taster.h"
#include "main.h"



uint8_t bing=0,bong=0;				// Deklaration von globalen Variablen die berall genutzt werden knnen, hier muss man aufpassen da mehrere Teile des Programms darauf zugreifen und den Inhalt verndern knnen, wenn man die bersicht verliert -> Problem.
uint8_t Schritt=0;
uint16_t Takt=0,Zahler=0;
uint8_t Freigabe=0;
uint16_t Auto_LED=0;

uint16_t Zuendpunkt1=0, Zuendpunkt2=0;

ISR(INT0_vect)						// Interrupt bei jedem zweiten Nulldurchgang ( einmal pro Vollwelle )
{
	Zahler++;
	
	if(Zahler>=Takt)				// In der variable Takt liegt der Teiler durch den die Netzfrequenz geteilt wird. Wertebereich 0 - 65535 (uint16)
	{	
		Zahler=0;					// Wenn Zhlerstand erreicht dann zurcksetzen
		if(Freigabe==1)				// Wenn Freigabe fr Automatikbetrieb vorhanden und Zhlerstand erreicht dann klingeln
			bing=1;
		Auto_LED=25;				// Auto LED ist an Wenn Variable >0 -> So gemacht damit die LED einen Moment leuchtet(25 Hauptprogramm durchlufe)
	}
	
	
	if(bing==1)						// Zustand hier bernehmen so das immer eine Vollwelle abgegeben wird und niemals eine Halbwelle
	{
		bong=1;
		bing=0;
	}
		
	TCNT1=0;						// Zhlwert des Timer1 zurcksetzen
	Schritt=0;						// Schrittkette zurcksetzen
	OCR1A=Zuendpunkt1;				// Timer zum znden fr die erste Halbwelle stellen
		
}

ISR(TIMER1_COMPA_vect)							// Dieser interrupt wird aufgerufen wenn der Zhlwert in TCNT1 den Vergleichswert in OCR1A erreicht.
{

	switch(Schritt)
	{
		case 0:									// Dieser Teil wird zum ersten Zndzeitpunkt aufgerufen
			if(bong==1)
				TriacEIN;
			Schritt=1;
			OCR1A = Triac_Abschaltpunkt1;		// Timer zum abschalten des Triacs stellen, die Triac Abschaltpunkte sind in der main.h definiert
			break;
		case 1:									// Dieser Teil wird zum ersten Abschaltpunkt aufgerufen
			TriacAUS;
			Schritt=2;
			OCR1A = Zuendpunkt2;				// Timer zum znden fr die zweite Halbwelle stellen
			break;
		case 2:									// Dieser Teil wird zum zweiten Zndzeitpunkt aufgerufen
			if(bong==1)
				TriacEIN;									
			Schritt=3;
			OCR1A = Triac_Abschaltpunkt2;		// Timer zum abschalten des Triacs stellen
			break;
		case 3:									// Dieser Teil wird zum zweiten Abschaltpunkt aufgerufen
			TriacAUS;
			Schritt=4;
			bong=0;								// Zyklus komplett
			break;
		default:
			break;
	}
}

int main(void)												// Haupt Programm, alles bis zur while(1) Schleife wird nur einmal ausgefhrt nach Reset des Controllers
{
	wdt_enable(WDTO_500MS);									// Watchdog Timer zurcksetzen
	Taster_init();											// Taster Routine initialisieren 
	ADC_Init();
	
	set_bit(DDRD,5);	// LED KEY0	"Einzelschlag"			// DataDirectionRegiser_portD -> DDRD setzen so das PD5 ein Ausgang wird
	set_bit(DDRD,6);	// LED KEY1 "Auto"
	set_bit(DDRD,7);	// Ausgang Optotriac
	
	set_bit(PORTD,5);	// LED KEY0	"Einzelschlag" an.
	
	EICRA=0b00000010;	// Fallende Flanke auf INT0 erzeugt interrupt		// Konfiguration des Interrupts fr die Nulldurchgangserkennung
	EIMSK=0b00000001;	// interrupt aktivieren
	
	TCCR1A=0b00000000;														// stellen von Timer 1 so das dieser mit 2MHz hochzhlt
	TCCR1B=0b00000010;	// Normal Modus clk/8
	OCR1A=50;			// 40000 -> 20ms									// wenn OCR1A erreicht wird dann wird der compare match interrupt ausgelst
	TIMSK1=0b00000010;	// interrupt bei OCR1A
   
	sei();																	// interrupts aktivieren
	
	uint16_t adc=0;															// Deklaration der variablen die im Hauptprogramm benutzt werden
	uint16_t adc2=0;
	uint16_t buf=0;
	uint8_t Automatik=0;
	
	float Puffer=0.0;
   
    while (1) 
    {			
		wdt_reset();														// Watchdog Timer zurcksetzen sonst resettet sich der Controller nach 0,5s selbst
		
		adc=ADC_Read_Avg(0,50);												// Den Analog-Digital Wandler fr Kanal 0 mit eiem Mittelwert ber 50 Werte auslesen, liefert 0-1023 fr 0V bis VCC(idR.5V)
		buf=13000-(adc*12);													// Den Zeitwert fr den Zndzeitpunkt berechnen, da der Timer mit 2MHz luft entspricht 10000 -> 5ms, Werte kleiner 500 oder grer 19500 machen keinen Sinn.
		if(buf>13000)														// Diese Berechnung ist optimiert fr die Klingel, willst man keinen Phasenanschnitt sondern nur voll wellen setzt man buf auf 500. Damit wird der Triac 0,25msec nach dem Nulldurchgang gezndet
			buf=13000;
		if(buf<1000)
			buf=1000;
		Zuendpunkt1=buf;													// 500 wre nahezu maximale Pulsbreite, 19500 wre nahezu minimale Pulsbreite
		Zuendpunkt2=buf+20000;												// +20000 = 10msec da die zweite Halbwelle ja 10msec auf die erste folgt

		
		adc2=ADC_Read_Avg(1,50);											// Den Analog-Digital Wandler fr Kanal 1 mit eiem Mittelwert ber 50 Werte auslesen, liefert 0-1023 fr 0V bis VCC(idR.5V)
				
		Puffer=(P1*pow(adc2,4))+(P2*pow(adc2,3))+(P3*pow(adc2,2))+(P4*adc2)+P5;			// Diese Funktion nhert das nicht lineare Verhalten von 1/x also dem Teiler etwas an so das man die einstelligen Werte des Teilers sinnvoll einstellen kann. 
																						// Sonst wre es sehr schwierig mit dem Poti das 0-1023 liefer z.B. 1 oder 2 einzustellen, mit dieser Funktion hat man einen Bereich von ungefhr 20 bei dem dann nur 0, 1, 2, etc. herauskommen.
		if(Puffer>350.0)																// Knnte man natrlich auch auf andere weise lsen, das hier ist sicher nicht die einfachste Version...
			Puffer=350.0;			// Begrenzen der Teilerwerte auf sinnvolle Werte
		if(Puffer<1.0)				
			Puffer=1.0;				// Kleiner 1 ist nicht sinnvoll da sonst Division durch null -> schlecht
				
		Takt=(uint16_t)Puffer;		

		
		if(get_key_press(1<<KEY0))	// Diese Funktionen werden wenn man den Taster drckt nur ein einziges mal ausgefhrt bis man ihn loslsst und erneut drckt
			bing=1;					// Sorgt fr einen einzelnen Puls am Ausgang
		
		if(get_key_press(1<<KEY1))	
			Automatik=0;			// Schaltet die Automatik aus
		
		if(get_key_state(1<<KEY0)&&get_key_state(1<<KEY1))
			Automatik=1;			// Wenn gleichzeitig beide Knpfe gedrckt werden wird die Automatische Puls Ausgabe aktiviert
		
		if(get_key_state(1<<KEY1)||(Automatik==1))
			Freigabe=1;				// Wenn man den Knopf fr die automatische Impulsausgabe gedrckt hlt oder Automatik auf 1 steht dann werden mit dem entsprechenden Teilerverhltnis Pulse ausgegeben. Das passiert in den beiden interrupt Routinen ganz oben.
		else
			Freigabe=0;
		

		
		if(Auto_LED>0)				// Hier werden die LEDs der Taster gesteuert, so das diese wahlweise blinken oder leuchten
		{
			Auto_LED--;				// Das ist ein Kniff so das die LED eine gewisse Mindestzeit leuchtet, ansonsten knnte man kaum sehen wenn die LED nur einen bruchteil einer Sekunde an ist.
			LED_AutoEIN;			// Die Funktion "LED_AutoEIN" ist in der main.h im solution explorer definiert, aus dem Grund das man direkt sieht was dieser Ausdruck bewirkt. Der compiler setzt an dieser Stelle einfach "set_bit(PORTD,6)" ein, ohne Liste welche Anschlsse was machen geht daraus nicht hervor was das genau macht.
			
			if(Automatik==1)
				LED_EinzelschlagAUS;		
		}else{
			LED_AutoAUS;
			if(Automatik==1)
				LED_EinzelschlagEIN;
		}
		
		if(Automatik==0)
			LED_EinzelschlagEIN;
		
    }
}

